/* Butuc Elena-Roxana
 * Grupa 335CA
 * Materia: Proiectarea cu Microprocesoare
 * Proiect "Joc de X si 0 cu scor si sunet"
 * Asistent: Dan Dragomir
 */

#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>

#define MAX_ITERATION 100000
#define F_CPU 16000000
#define MAX_NEG -10

#define MORSE_LUNG		3
#define MORSE_SCURT		1

#define NUM_NOTE		8
#define DURATA_NOTA		100

float frecventa_nota[8] = {
	261.63, 293.66, 329.63, 349.23, 392.0, 440.0, 493.88, 523.25
};

int tip_morse = 0;
int scor_red = 0;
int scor_green = 0;
int prev_scor_red = 0;
int prev_scor_green = 0;

/* 
	
	--------------------------------
	
	Matrice red
	
	PD1 PA7 PC6
	
	PA4 PA6 PC7
	
	PD0 PA3 PA5
	
	--------------------------------
	Matrice green                           
	
	PA0 PC5 PC0
	 
	PA1 PC4 PC1
	
	PA2 PC3 PC2
	
	------------------------------
	
	
	Pini pentru butoane:
	
	btn. jos                         PB0
	btn. stanga                      PB1

	btn. centru                      PB3
	btn. dreapta                     PB4
	btn. sus                         PB2
	
	-------------------------------
	

	Pin pentru difuzor 
	PD4	

	GND
	PD-

	Scor verde
	PD5 PD6 PD7

	Scor rosu
	PB5 PB6 PB7


*/


typedef struct _pin_details {
	int pin;
	int port; // 1 pentru PORTA, 2 pentru PORTB, 3 pentru PORTC, 4 pentru PORTD
} PINI;

typedef struct _winner_details {
	int won; //1 daca a castigat
	int line;
	int col;
	int diag;
	int equal; //daca a facut egalitate
	int scor;

} _WINNER;

// Corespondenta matrice cu pini de pe microcontroller
PINI check_led(int line, int col, int next_move)
{
	PINI s_pin;
	int player_1;
	int player_2;
	if ((next_move % 2) == 0) {
		player_1 = 1;
		player_2 = 0;
	} else {
		player_1 = 0;
		player_2 = 1;
	}
		
	
	//Red and Green
	
	if ((line == 0) && (col == 0)){
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PD1;
			s_pin.port = 4;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PA0;
			s_pin.port = 1;
		}
		return s_pin;
	} 
	
	if ((line == 0) && (col == 1)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PA7;
			s_pin.port = 1;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PC5;
			s_pin.port = 3;
		}
		return s_pin;
	}
	
	if ((line == 0) && (col == 2)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PC6;
			s_pin.port = 3;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PC0;
			s_pin.port = 3;
		}
		return s_pin;
	}
	
	if ((line == 1) && (col == 0)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PA4;
			s_pin.port = 1;
		} else if (player_1 == 0 && player_2 == 1) {//player_2 -> culoarea green
			s_pin.pin = PA1;
			s_pin.port = 1;
		}
		return s_pin;
	}
	

	if ((line == 1) && (col == 1)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PA6;
			s_pin.port = 1;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PC4;
			s_pin.port = 3;
		}
		return s_pin;
	}
	
	if ((line == 1) && (col == 2)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PC7;
			s_pin.port = 3;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PC1;
			s_pin.port = 3;
		}
		return s_pin;
	}
	
	if ((line == 2) && (col == 0)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PD0;
			s_pin.port = 4;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PA2;
			s_pin.port = 1;
		}
		return s_pin;
	}
	
	if ((line == 2) && (col == 1)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PA3;
			s_pin.port = 1;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PC3;
			s_pin.port = 3;
		}
		return s_pin;
	}
	
	if ((line == 2) && (col == 2)) {
		if (player_1 == 1 && player_2 == 0) { //player_1 -> culoarea red
			s_pin.pin = PA5;
			s_pin.port = 1;
		} else if (player_1 == 0 && player_2 == 1) { //player_2 -> culoarea green
			s_pin.pin = PC2;
			s_pin.port = 3;
		}
		return s_pin;
	}
	
	
	// nu e selectat nimic
	s_pin.pin = -1;
	s_pin.port = -1;
	return s_pin;

}

// Inversam startea ledului
void led_light(PINI sw_color_led) {

	switch (sw_color_led.port) {
	case 1:
		PORTA ^= ( 1 << sw_color_led.pin);
		break;
	case 2:
		PORTB ^= ( 1 << sw_color_led.pin);
		break;
	case 3:
		PORTC ^= ( 1 << sw_color_led.pin);
		break;
	case 4:
		PORTD ^= ( 1 << sw_color_led.pin);
		break;
	}

}

/* Genereaza sunetul pentru tipul de ton folosit: lung sau scurt. */
void speaker_morse(int tip_morse) {
	/*
	 * durata, in milisecunde, in functie de tipul de ton folosit: 50 ms
	 * pentru un ton scurt, 150 ms pentru un ton lung
	 */
	int durata = 50 * tip_morse;
	int i;

	/* alternam starea pinului PD4 la intervale de timp foarte mici */
	for (i = 0; i < durata; ++i) {
		PORTD ^= (1 << PD4);
		_delay_ms(3);
	}
}

/*
 * Genereaza tonuri pentru codul Morse
 */
void sound_equal(void) {
	
	/* setez pinul PD4 ca iesire */
	DDRD |= (1 << PD4);

	int i = 0;
	while (i < 5) {
		if (tip_morse == 3)
			speaker_morse(MORSE_LUNG);
		else
			speaker_morse(MORSE_SCURT);
		PORTD &= ~(1 << PD4);
		_delay_ms(50);
		i++;
	}
	
	/* opresc difuzorul */
	PORTD &= ~(1 << PD4);
}

/* Genereaza o nota */
void nota_reda(int nota) {
	float perioada_nota = 100000 / frecventa_nota[nota];
	int i, j;

	for (i = 0; i < DURATA_NOTA; ++i) {
		PORTD |= (1 << PD4);

		/*
		 * nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".

		 */
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);
		
		/* opresc difuzorul */
		PORTD &= ~(1 << PD4);

		/*
		 * nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0 ; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}
}

void sound_winner(void) {

	/* setez pinul PD4 ca iesire. */
	DDRD |= (1 << PD4);
	
	int i;

	for (i = 0; i < 11; i++) {
		nota_reda(7);
		_delay_ms(50);
	}
	
	/* opresc difuzorul */
	PORTD &= ~(1 << PD4);
}

void scor(void)
{
	/* setez pinii pentru scorul rosu ca iesire */
	DDRD |= (1 << PD5);
	DDRD |= (1 << PD6);
	DDRD |= (1 << PD7);

	PORTD &= ~(1 << PD5);
	PORTD &= ~(1 << PD6);
	PORTD &= ~(1 << PD7);
	
	/* setez pinii pentru scorul verde ca iesire */
	DDRB |= (1 << PB5);
	DDRB |= (1 << PB6);
	DDRB |= (1 << PB7);

	PORTB &= ~(1 << PB5);
	PORTB &= ~(1 << PB6);
	PORTB &= ~(1 << PB7);
	
	PINI r_pin;
	PINI g_pin;
	
	if (scor_green != 0) {
		switch(scor_green){
		case 1:
			g_pin.pin = PD7;
			g_pin.port = 4;
			led_light(g_pin);
			break;
		case 2:
			g_pin.pin = PD6;
			g_pin.port = 4;
			led_light(g_pin);
			break;
		case 3:
			g_pin.pin = PD6;
			g_pin.port = 4;
			led_light(g_pin);
			
			g_pin.pin = PD7;
			g_pin.port = 4;
			led_light(g_pin);
			break;
		case 4:
			g_pin.pin = PD5;
			g_pin.port = 4;
			led_light(g_pin);
			break;
		case 5:
			g_pin.pin = PD5;
			g_pin.port = 4;
			led_light(g_pin);
	
			g_pin.pin = PD7;
			g_pin.port = 4;
			led_light(g_pin);
			break;
		case 6:
			g_pin.pin = PD5;
			g_pin.port = 4;
			led_light(g_pin);
	
			g_pin.pin = PD6;
			g_pin.port = 4;
			led_light(g_pin);
			break;
		case 7:
			g_pin.pin = PD5;
			g_pin.port = 4;
			led_light(g_pin);
	
			g_pin.pin = PD6;
			g_pin.port = 4;
			led_light(g_pin);

			g_pin.pin = PD7;
			g_pin.port = 4;
			led_light(g_pin);
			break;
				
		}
		
	}
	
	if ( scor_red != 0) {
		switch(scor_red){
		case 1:
			r_pin.pin = PB7;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		case 2:
			r_pin.pin = PB6;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		case 3:
			r_pin.pin = PD6;
			r_pin.port = 2;
			led_light(r_pin);
			
			r_pin.pin = PB7;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		case 4:
			r_pin.pin = PB5;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		case 5:
			r_pin.pin = PB5;
			r_pin.port = 2;
			led_light(r_pin);
	
			r_pin.pin = PB7;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		case 6:
			r_pin.pin = PB5;
			r_pin.port = 2;
			led_light(r_pin);
	
			r_pin.pin = PB6;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		case 7:
			r_pin.pin = PB5;
			r_pin.port = 2;
			led_light(r_pin);
	
			r_pin.pin = PB6;
			r_pin.port = 2;
			led_light(r_pin);
			
			r_pin.pin = PB7;
			r_pin.port = 2;
			led_light(r_pin);
			break;
		}
	}

}

// Verificam daca avem castigator
_WINNER check_w(int mat_led[3][3])
{
	int once = 0;
	int linie, coloana, diagonala1, diagonala2;
	diagonala1 = 0;
	diagonala2 = 0;

	int i, j;
	_WINNER aux;
	
	aux.won = 0;
	aux.line = -10;
	aux.col = -10;
	aux.diag = -10;
	aux.equal = 0;

	// verific liniile
	for (linie = 0; linie< 3; linie++) {
		// jucator cu led rosu
		if ( mat_led[linie][0] == 1 && mat_led[linie][1] == 1
				&& mat_led[linie][2] == 1) {
			aux.won = 1;
			aux.line = linie;
			scor_red++;
			scor();
			return aux;
		}
		//jucator cu led verde
		if ( mat_led[linie][0] == 2 && mat_led[linie][1] == 2
				&& mat_led[linie][2] == 2) {
			aux.won = 1;
			aux.line = linie;
			scor_green++;
			scor();
			return aux;
		}
	}
	
	//verific coloanele
	for (coloana = 0; coloana < 3; coloana++) {
		// jucator cu led rosu
		if ( mat_led[0][coloana] == 1 && mat_led[1][coloana] == 1
				&& mat_led[2][coloana] == 1) {
			aux.won = 1;
			aux.col = coloana;
			scor_red++;
			return aux;
		}
		// jucator cu led verde
		if ( mat_led[0][coloana] == 2 && mat_led[1][coloana] == 2
				&& mat_led[2][coloana] == 2) {
			aux.won = 1;
			aux.col = coloana;
			scor_green++;
			return aux;
		}
	}
	
	// verific diagonala1 (diagonala secundara)

	// jucator cu led rosu
	if ( mat_led[diagonala1][2] == 1 && mat_led[diagonala1 + 1][1] == 1
				&& mat_led[diagonala1 + 2][0] == 1) {
		aux.won = 1;
		aux.diag = 200;
		scor_red++;
		return aux;
	}
	// jucator cu led verde
	if ( mat_led[diagonala1][2] == 2 && mat_led[diagonala1 + 1][1] == 2
				&& mat_led[diagonala1 + 2][0] == 2) {
		aux.won = 1;
		aux.diag = 200;
		scor_green++;
		return aux;
	}
	
	// verific diagonala2 (diagonala principala)

	// jucator cu led rosu
	if ( mat_led[diagonala2][0] == 1 && mat_led[diagonala2 + 1][1] == 1
				&& mat_led[diagonala2 + 2][2] == 1) {
		aux.won = 1;
		aux.diag = 100;
		scor_red++;
		return aux;
	}
	// jucator cu led verde
	if ( mat_led[diagonala2][0] == 2 && mat_led[diagonala2 + 1][1] == 2
				&& mat_led[diagonala2 + 2][2] == 2) {
		aux.won = 1;
		aux.diag = 100;
		scor_green++;
		return aux;
	}

	// verificam daca a fost egalitate
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			if ((mat_led[i][j] == 0) && (once == 0)) {
				once = 1;
			}
		}
	}
	
	if (once == 0) {
		
		aux.won = 3;
		aux.line = -10;
		aux.col = -10;
		aux.diag = -10;
		aux.equal = 1;
		return aux;
	}
		
	aux.won = 0;
	aux.line = -10;
	aux.col = -10;
	aux.diag = -10;
	aux.equal = 0;
	return aux;
		
}

int main(){
	
	
	// Setăm toti pinii portului A, B, C si D că pini de iesire
	DDRB = 0xFF;
	DDRA = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFF;
	
	
	/* Pentru butoane test */
	
	// Buton sus
	 DDRB &= ~(1 << PB2);  // pinul PB2 va fi input
	 PORTB |= (1 << PB2);  // activam rezistenta de pull-up
	 
	 // Buton stanga
	 DDRB &= ~(1 << PB1);  // pinul PB1 va fi input
	 PORTB |= (1 << PB1);  // activam rezistenta de pull-up
	 
	 // Buton dreapta
	 DDRB &= ~(1 << PB4);  // pinul PB4 va fi input
	 PORTB |= (1 << PB4);  // activam rezistenta de pull-up
	 
	  // Buton centru
	 DDRB &= ~(1 << PB3);  // pinul PB3 va fi input
	 PORTB |= (1 << PB3);  // activam rezistenta de pull-up
	 
	 // Buton jos
	 DDRB &= ~(1 << PB0);  // pinul PB0 va fi input
	 PORTB |= (1 << PB0);  // activam rezistenta de pull-up
	 
			
	int i, j;
	int mat_led[3][3];
	int next_move;
	int line = 0, col = 0;
	int once = 0;
	_WINNER w;
	PINI ret_pd;
	
	int nr_jocuri = 0;
	
	while (nr_jocuri < 9) {

		if (prev_scor_red != scor_red)
			next_move = 1;
		else
			next_move = 0;

		// Initializare structura winner
		w.won = 0;
		w.line = -10;
		w.col = -10;
		w.diag = -10;
		w.equal = 0;
	
		// Initializare matrice de leduri
		for (i = 0; i < 3; i++) {
			for (j = 0; j < 3; j++) {
				mat_led[i][j] = 0;
			}
		}
	
	
		while(1) {
	
			line = 0;
			col = 0;
			once = 0;
		
			if (w.won != 0) {
				break;
			}
		
			// Verificam led liber
			for (i = 0; i < 3; i++) {
				for (j = 0; j < 3; j++) {
					if ((mat_led[i][j] == 0) && (once == 0)) {
						line = i;
						col = j;
						once = 1;
					}
				}
			}
		
		
			// Determinam pinul cu ledul liber
			ret_pd = check_led (line, col, next_move);
		

			// Aprindem ledul
			led_light(ret_pd);

			prev_scor_red = scor_red;
			prev_scor_green = scor_green;

			_delay_ms(500);
		
		
				// Miscare stanga, dreapta, sus, jos, selectie
				while (1) {
				
					// buton dreapta
					if (!(PINB & (1 << PB4))) {
					
							// stingem ledul de unde a venit
							led_light(ret_pd);
						
							// iteram in dreapta pana gasim un led neaprins
							do {
						
								// up pe line
								if (col == 2) {
									line = (line + 1) % 3;
								}
						
								// up pe coloana
								col = (col + 1) % 3;
						
							} while (mat_led[line][col] != 0);
						
							// aprind noul led
							ret_pd = check_led (line, col, next_move);
							led_light(ret_pd);
						
							_delay_ms(500);
					}
				
					// buton stanga
					if (!(PINB & (1 << PB1))) {
				
					
							// stingem ledul de unde a venit
							led_light(ret_pd);
						
						
							// iteram in stanga pana gasim un led neaprins
							do {
							// down pe linie
							if (col == 0) {
								line = line - 1;
								if (line < 0) line = 2;
								line= line % 3;
							}
						
							// down pe coloana
							col = col - 1;
							if (col < 0) col = 2;
							col = col % 3;
						
							} while (mat_led[line][col] != 0);
						
							// aprind noul led
							ret_pd = check_led (line, col, next_move);
							led_light(ret_pd);
						
							_delay_ms(500);
					}
				
					// buton sus
					 if (!(PINB & (1 << PB2))) {
					
							// stingem ledul de unde a venit
							led_light(ret_pd);
						
							// iteram in sus pana gasim un led neaprins
							do {
						
							// down pe coloana
							if (line == 0) {
								col = col - 1;
								if (col < 0) 
									col = 2;
								col = col % 3;
							}
						
							// down pe linie
							line = line - 1;
							if (line < 0)
								line = 2;
							line = line % 3;
						
							} while (mat_led[line][col] != 0);
						
							// aprind noul led
							ret_pd = check_led (line, col, next_move);
							led_light(ret_pd);
						
							_delay_ms(500);
					
					 }
					 
					 // buton jos
					 if (!(PINB & (1 << PB0))) {
					 
							// stingem ledul de unde a venit
							led_light(ret_pd);
						
							// iteram in jos pana gasim un led neaprins
							do {
						
							// up pe coloana
							if (line == 2) {
								col = (col + 1) % 3;
							}
						
							// up pe linie
							line = (line + 1) % 3;
						
							} while (mat_led[line][col] != 0);
						
							// aprindem noul led
							ret_pd = check_led (line, col, next_move);
							led_light(ret_pd);
						
							_delay_ms(500);
					 }
				
					// buton centru (de selectie)
					if (!(PINB & (1 << PB3))) {
					
						// selectie led pentru jucator red
						if ((next_move % 2) == 0) {
							mat_led[line][col] = 1;
						}
						// selectie led pentru jucator green
						else {
							mat_led[line][col] = 2;
						}
					
						w = check_w(mat_led);
					
						next_move++;
						break;
					}
					scor();
				
				}//de la while
			}
	
	
		// evidentiem castigatorul
		int f = 0;
		while (f < 3) {
	
			next_move = w.won;
		
			// daca s-a completat o linie
			if (w.line != -10) {
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (w.line, i, next_move);
					led_light(ret_pd);
				}
				_delay_ms(500);

				for (i = 0; i < 3; i++) {
					ret_pd = check_led (w.line, i, next_move);
					led_light(ret_pd);
				}
			
				tip_morse = 3;
				sound_winner();
				_delay_ms(500);
		
			}
		
			// daca s-a completat o coloana
			if (w.col != -10) {
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (i, w.col, next_move);
					led_light(ret_pd);
				}
			
				_delay_ms(500);
			
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (i, w.col, next_move);
					led_light(ret_pd);
				}
			
				tip_morse = 3;
				sound_winner();
				_delay_ms(500);
		
			}
		
			// daca s-a completat diagonala principala
			if (w.diag == 100) {
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (i, i, next_move);
					led_light(ret_pd);
				}	
				_delay_ms(500);
				
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (i, i, next_move);
					led_light(ret_pd);
				}

				tip_morse = 3;
				sound_winner();	
				_delay_ms(500);
			}
		
			// daca s-a completat diagonala secundara
			if (w.diag == 200) {
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (i, 3-i-1, next_move);
					led_light(ret_pd);
				}	
				_delay_ms(500);
				
				for (i = 0; i < 3; i++) {
					ret_pd = check_led (i, 3-i-1, next_move);
					led_light(ret_pd);
				}
			
				tip_morse = 3;
				sound_winner();	
				_delay_ms(500);
			}
		
			// egalitate
			if (w.equal == 1) {
				tip_morse = 1;
				sound_equal();			
				_delay_ms(500);		
			}
			f++;
		}
	
		scor();

		// sting toata matricea de led-uri
		PORTD &= ~(1 << PD1); PORTA &= ~(1 << PA0);
		PORTA &= ~(1 << PA4); PORTA &= ~(1 << PA1);
		PORTD &= ~(1 << PD0); PORTA &= ~(1 << PA2);
		PORTA &= ~(1 << PA7); PORTC &= ~(1 << PC5);
		PORTA &= ~(1 << PA6); PORTC &= ~(1 << PC4);
		PORTA &= ~(1 << PA3); PORTC &= ~(1 << PC3);
		PORTC &= ~(1 << PC6); PORTC &= ~(1 << PC0);
		PORTC &= ~(1 << PC7); PORTC &= ~(1 << PC1);
		PORTA &= ~(1 << PA5); PORTC &= ~(1 << PC2);

		nr_jocuri++;
	}
	return 0;
}
